# This file is part of the lwbc cataloguing program
# Copyright © 2012 William Astle
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANT; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
package LWBC;
use strict;
use Carp;

#use LWBC::Driver;

sub new
{
	my $class = shift;
	my %opts = @_;
	my $self = {};
	bless($self,$class);
	
	$self->{'_conn'} = undef;
	if (defined($opts{'driver'}))
	{
		croak "Connection failed" unless $self->connect(%opts);
	}
	return $self;
}

sub connect
{
	my ($self, %opts) = @_;
	
	croak "Already connected" if defined($self->{'_conn'});
	my $drvr = 'LWBC::Driver::' . $opts{'driver'};
	eval "require $drvr";
	croak "Unknown driver " . $opts{'driver'} if $@ ne '';
	$self->{'_conn'} = $drvr->new(%opts);
	return 1;
}

sub fetch_work
{
	my ($self, $key) = @_;
	croak "Not connected" unless defined($self->{'_conn'});
	return $self->{'_conn'}->fetch_work($key);
}

sub fetch_person
{
	my ($self, $key) = @_;
	croak "Not connected" unless defined($self->{'_conn'});
	return $self->{'_conn'}->fetch_person($key);
}

sub search_work
{
	my ($self, $search) = @_;
	croak "Not connected" unless defined($self->{'_conn'});
	return $self->{'_conn'}->search_work($search);
}

sub save_work
{
	my ($self, $rec) = @_;
	croak "Not connected" unless defined($self->{'_conn'});
	return $self->{'_conn'}->save_work($rec);
}

sub search_person
{
	my ($self, $search) = @_;
	croak "Not connected" unless defined($self->{'_conn'});
	return $self->{'_conn'}->search_person($search);
}

sub add_person
{
	my ($self, $rec) = @_;
	croak "Not connected" unless defined($self->{'_conn'});
	return $self->{'_conn'}->add_person($rec);
}

1;
