#!/usr/bin/perl -w
# This file is part of the lwbc cataloguing program
# Copyright © 2012 William Astle
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANT; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
use utf8;
use strict;

use Data::Dumper;

####FIXME: make this more sensible and more portable
use File::Basename;
use lib dirname($0);

use LWBC;
use Glib;
use Gtk2 qw(-init);
use Time::HiRes qw(usleep);

# global variables
my $db = undef;
my %state = ();

# read configuration file and various other gimmicks here
my %config;
eval
{
	%config = &read_cf;
	if (defined($config{'db'}{'type'}))
	{
		$db = LWBC->new(
			'driver' => $config{'db'}{'type'},
			'dbhost' => $config{'db'}{'host'},
			'dbname' => $config{'db'}{'name'},
			'dbuser' => $config{'db'}{'user'},
			'dbpass' => $config{'db'}{'pass'},
			'dbport' => $config{'db'}{'port'}
		);
		if (!defined($db))
		{
			die "Cannot open database";
		}
	}
};
my $openerr = $@;

my $mainwindow = Gtk2::Window->new;

$mainwindow->signal_connect(destroy => sub { Gtk2->main_quit;});
$mainwindow->set_default_size(640, 480);

&update_status;

if ($openerr ne '')
{
	my $dlg = Gtk2::MessageDialog->new($mainwindow, [], 'error', 'yes-no', "Error with configured database connection information: $openerr\n\nStart up unconnected?");
	my $res = $dlg->run;
	undef $config{'db'};
	$dlg->destroy;
	exit if $res eq 'no';
}

# create menubar
my $menubar = Gtk2::MenuBar->new;
my $fileitem = Gtk2::MenuItem->new('File');
$menubar->append($fileitem);

my $filemenu = Gtk2::Menu->new;
$fileitem->set_submenu($filemenu);

my $fileopen = Gtk2::MenuItem->new('Open');
$fileopen->signal_connect(activate => sub {
	if ($state{'newshown'})
	{
		$state{'newdlg'} -> present;
		return;
	}
	
	my $l;
	my $newdlg = Gtk2::Dialog->new('Open Database', $mainwindow, 'destroy_with_parent', 'Cancel' => 'cancel', 'Open' => 'accept');
	my $tbl = Gtk2::Table->new(3,2,0);
	$l = Gtk2::Label->new('Database Type:');
	$l->set_alignment(1, 0.5);
	$tbl->attach($l,0, 1, 0, 1, [ 'fill', 'shrink' ], ['shrink'], 0, 0);
	my $typebox = Gtk2::ComboBox->new_text;
	$tbl->attach($typebox, 1, 2, 0, 1, [ 'expand', 'fill' ], [ 'shrink' ], 0, 0);
	$typebox->append_text('MongoDB');
	$typebox->set_active(0);
	$l = Gtk2::Label->new('Hostname:');
	$l->set_alignment(1, 0.5);
	$tbl->attach($l, 0, 1, 1, 2, [ 'fill' ], [ ], 0, 0);
	my $hostbox = Gtk2::Entry->new;
	$tbl->attach($hostbox, 1, 2, 1, 2, [ 'expand', 'fill' ], [ 'shrink' ], 0, 0);
	$l = Gtk2::Label->new('Data Name:');
	$l -> set_alignment(1, 0.5);
	$tbl->attach($l, 0, 1, 2, 3, [ 'fill' ], [ 'shrink' ], 0, 0);
	my $namebox = Gtk2::Entry->new;
	$tbl->attach($namebox, 1, 2, 2, 3, [ 'expand', 'fill' ], [ 'shrink' ], 0, 0);
	$l = Gtk2::Label->new('Username:');
	$l->set_alignment(1, 0.5);
	$tbl->attach($l, 0, 1, 3, 4, [ 'fill' ], [ 'shrink' ], 0, 0);
	my $unbox = Gtk2::Entry->new;
	$tbl->attach($unbox, 1, 2, 3, 4, [ 'expand', 'fill' ], [ 'shrink' ], 0, 0);
	$l = Gtk2::Label->new('Password:');
	$l->set_alignment(1, 0.5);
	$tbl->attach($l, 0, 1, 4, 5, [ 'fill' ], [ 'shrink' ], 0, 0);
	my $pwbox = Gtk2::Entry->new;
	$tbl->attach($pwbox, 1, 2, 4, 5, [ 'expand', 'fill' ], [ 'shrink' ], 0, 0);
	$newdlg->get_content_area->pack_start($tbl, 1, 1, 0);
	$newdlg->signal_connect(response => sub {
		my ($dlg, $evt) = @_;
		if ($evt eq 'accept')
		{
			undef $db;
			undef $config{'db'};
			my $type = $typebox->get_active_text;
			if ($type eq 'MongoDB')
			{
				$type = 'Mongo';
			}
			my $host = $hostbox->get_text;
			my $port;
			if ($host =~ /^(.+):(.+)$/)
			{
				$host = $1;
				$port = $2;
			}
			my $name = $namebox->get_text;
			my $user = $unbox->get_text;
			my $pass = $pwbox->get_text;
			my $ndb;
			eval
			{
				$ndb = LWBC->new(
					'driver' => $type,
					'dbhost' => $host,
					'dbport' => $port,
					'dbname' => $name,
					'dbuser' => $user,
					'dbpass' => $pass
				);
			};
			if ($@ eq '')
			{
				$db = $ndb;
				$config{'db'} = {
					'type' => $type,
					'host' => $host,
					'port' => $port,
					'user' => $user,
					'pass' => $pass,
					'name' => $name
				};
			}
			else
			{
				my $dlg = Gtk2::MessageDialog->new($mainwindow, [], 'error', 'ok', "Error connecting to database: $@");
				my $res = $dlg->run;
				$dlg->destroy;
				return;
			}
			&create_people_model;
			&create_work_model;
			&update_status;
		}
		$newdlg->destroy;
		undef $state{'newdlg'};
		$state{'newshown'} = 0;
	});
	$newdlg -> show_all;
	$state{'newshown'} = 1;
	$state{'newdlg'} = $newdlg;
});
$filemenu->append($fileopen);

my $fileclose = Gtk2::MenuItem->new('Close');
$fileclose->signal_connect(activate => sub {
	undef $db;
	undef $config{'db'};
	&update_status;
});

$filemenu->append($fileclose);

my $savepref = Gtk2::MenuItem->new('Save State');
$savepref->signal_connect(activate => sub {
	eval
	{
		&write_cf(%config);
	};
	if ($@ ne '')
	{
		my $dlg = Gtk2::MessageDialog->new($mainwindow, [], 'error', 'ok', "Error saving config state: $@");
		my $res = $dlg->run;
		$dlg->destroy;
	}
});
$filemenu->append($savepref);

my $filequit = Gtk2::MenuItem->new('Quit');
$filequit->signal_connect(activate => sub { Gtk2->main_quit; });
$filemenu->append($filequit);

my $edititem = Gtk2::MenuItem->new('Edit');
$menubar->append($edititem);



my $editmenu = Gtk2::Menu->new;
$edititem->set_submenu($editmenu);

my $editnew = Gtk2::MenuItem->new('Add Item');
$editnew->signal_connect(activate => sub {
	# create an empty item edit screen
	# empty parens are REQUIRED here
	&show_work_detail();
});

$editmenu->append($editnew);

my $helpitem = Gtk2::MenuItem->new('Help');
$menubar->append($helpitem);

my $helpmenu = Gtk2::Menu->new;
$helpitem->set_submenu($helpmenu);

my $helpabout = Gtk2::MenuItem->new('About');
$helpabout->signal_connect(activate => sub {
	if ($state{'aboutshown'})
	{
		$state{'aboutdlg'} -> show_all;
		$state{'aboutdlg'} -> present;
		return;
	}
	my $about = Gtk2::AboutDialog->new;
	$about->set_program_name('Lost Wizard Book Catalogue');
	$about->set_version('Pre-beta');
	$about->set_copyright('Copyright © 2012 William Astle');
	$about->set_comments('A book (and other stuff) cataloguing program.');
	$about->set_transient_for($mainwindow);
	$about->set_modal(0);
	$about->set_skip_taskbar_hint(1);
	$about->set_skip_pager_hint(1);
	$about->signal_connect(response => sub { $state{'aboutdlg'}->hide_all if $state{'aboutshown'} });
	$about->set_type_hint('utility');
	$about->show_all;
	$state{'aboutshown'} = 1;
	$state{'aboutdlg'} = $about;
});
$helpmenu->append($helpabout);

my $layout = Gtk2::VBox->new;
$layout->pack_start($menubar, 0, 0, 0);

my $toolbar = Gtk2::Toolbar->new;
$toolbar->set_style("GTK_TOOLBAR_TEXT");
my $tbutton = Gtk2::ToolButton->new(undef,"Add Item");
$tbutton->signal_connect(clicked => sub {
	# create an empty item edit screen
	# empty parens are REQUIRED here
	&show_work_detail();
});
$toolbar->insert($tbutton, -1);

$layout->pack_start($toolbar, 0, 0, 0);

$mainwindow->add($layout);

&show_main;
&create_people_model;

$mainwindow->show_all;

Gtk2->main;

###FIXME: make this portable to non-unix
sub write_cf
{
	my %config = @_;
	my $fn;
	
	$fn = $config{'meta'}{'filename'};
	if (!defined($fn))
	{
		if (defined($ENV{'HOME'}) && -d $ENV{'HOME'})
		{
			my $d = $ENV{'HOME'} . '/.lwbc';
			if (! -d $d)
			{
				mkdir $d or die "Cannot create config directory ($d): $!";
			}
			if (-d $d)
			{
				$fn = $d . '/lwbcrc';
			}
			else
			{
				$fn = '.lwbcrc';
			}
		}
		else
		{
			$fn = '.lwbcrc';
		}
	}
	open my $fh, '>', $fn or die "Cannot save options ($fn): $!";
	delete $config{'meta'}{'filename'};
	my $s;
	my $k;
	foreach $s (keys %config)
	{
		print $fh "[$s]\n";
		foreach $k (keys %{ $config{$s} })
		{
			print $fh "$k=" . $config{$s}{$k} . "\n" if defined($config{$s}{$k});
		}
		print $fh "\n";
	}	
	close $fh;
}

####FIXME: make this portable to non-unix
sub read_cf
{
	# read configuration file and various other gimmicks here
	my %config;
	my $fn;

	my $fh;
	if (-r '.lwbcrc')
	{
		open $fh, '<', '.lwbcrc' or die "Cannot read .lwbcrc: $!";
		$fn = '.lwbcrc';
	}
	elsif (-r $ENV{'HOME'} . '/.lwbc/lwbcrc')
	{
		$fn = $ENV{'HOME'} . '/.lwbc/lwbcrc';
		open $fh, '<', $fn or die "Cannot read $fn: $!";
	}
	else
	{
		return %config;
	}

	my $l;
	my $sect = '';
	while (defined($l = <$fh>))
	{
		chomp $l;
		next if ($l =~ /^\s*$/);
		next if ($l =~ /^\s*\#/);
		if ($l =~ /^\s*\[(.+)\]/)
		{
			$sect = $1;
			next;
		}
		my ($k, $v) = split /=/, $l, 2;
		$config{$sect}{$k} = $v;
	}
	close $fh;
	$config{'meta'}{'filename'} = $fn;
	return %config;
}

sub update_status
{
	$state{'title'} = 'LWBC';
	if (defined($config{'db'}{'type'}))
	{
		$state{'title'} .= ' - ' . $config{'db'}{'type'} . ':' . $config{'db'}{'host'} . '/' . $config{'db'}{'name'};
	}

	$mainwindow -> set_title($state{'title'});
}

sub create_work_model
{
	my $blm;
	if (defined($state{'booklistmodel'}))
	{
		$blm = $state{'booklistmodel'};
		$blm->clear;
	}
	else
	{
		$blm = Gtk2::ListStore->new('Glib::String', 'Glib::String', 'Glib::String', 'Glib::String', 'Glib::Scalar', 'Glib::String');
		$state{'booklistmodel'} = $blm;
	}

	if (defined($db))
	{
		my $result = $db->search_work;
		my $i;
		my $i2;
		foreach $i (@$result)
		{
			$i2 = $db->fetch_work($i);
			my $t1 = $i2->{'title'};
			my $a = '';
			my $a2;
			my $ask = '';
			foreach $a2 (@{ $i2->{'contribs'} })
			{
				$a .= "\n" if $a ne '';
				$a .= $a2->{'name'};
				if ($a2->{'type'} ne 'author')
				{
					$a .= ' (' . $a2->{'type'} . ')';
				}
				$ask .= $a2->{'sortkey'} . "\n";
			}
			if (!defined($i2->{'copies'}) || !defined($i2->{'copies'}->[0]))
			{
				my $iter = $blm->append;
				$blm->set($iter, 0, $t1, 1, $a, 2, '<no copy>', 3, '<no copy>', $i2->{'id'}, 5, $ask);
			}
			else
			{
				foreach $a2 (@{ $i2->{'copies'} })
				{
					my $iter = $blm->append;
					$blm->set($iter, 0, $t1, 1, $a, 2, $a2->{'isbn'}, 3, $a2->{'lwid'}, 4, $i2->{'id'}, 5, $ask);
				}
			}
		}
	}
}

# shows the "main list" screen
sub show_main
{
	# create the main book list
	&create_work_model;
	my $booklist = Gtk2::TreeView->new($state{'booklistmodel'});
	
	my $render = Gtk2::CellRendererText->new;
	my $column = Gtk2::TreeViewColumn->new_with_attributes('Title', $render, 'text', 0);
	$column->set_sort_column_id(0);
	$column->set_resizable(1);
	$booklist->append_column($column);
	
	$render = Gtk2::CellRendererText->new;
	$column = Gtk2::TreeViewColumn->new_with_attributes('Author', $render, 'text', 1);
	$column->set_sort_column_id(5);
	$column->set_resizable(1);
	$booklist->append_column($column);
	
	$render = Gtk2::CellRendererText->new;
	$column = Gtk2::TreeViewColumn->new_with_attributes('ISBN', $render, 'text', 2);
	$column->set_sort_column_id(2);
	$column->set_resizable(1);
	$booklist->append_column($column);
	
	$render = Gtk2::CellRendererText->new;
	$column = Gtk2::TreeViewColumn->new_with_attributes('LWID', $render, 'text', 3);
	$column->set_sort_column_id(3);
	$column->set_resizable(1);
	$booklist->append_column($column);
	
	my $scw = Gtk2::ScrolledWindow->new;
	$layout->pack_start_defaults($scw);
	
	$scw->add($booklist);

	$state{'booklist'} = $booklist;
	
	# now attach the "edit" signal
	$booklist->signal_connect(row_activated => sub {
		my ($tv, $p, $c, $d) = @_;
		my $iter = $state{'booklistmodel'}->get_iter($p);
		my $val = $state{'booklistmodel'}->get_value($iter, 4);
		&show_work_detail($val);
	});
}

sub create_people_model
{
	# create the main author list
	# 0 => Name, 1 => sortkey, 2 => id
	my $peoplelistmodel;
	if (defined($state{'peoplelistmodel'}))
	{
		$peoplelistmodel = $state{'peoplelistmodel'};
		$peoplelistmodel->clear;
	}
	else
	{
		$peoplelistmodel = Gtk2::ListStore->new('Glib::String', 'Glib::String', 'Glib::Scalar');
		$state{'peoplelistmodel'} = $peoplelistmodel;
	}
	
	if (defined($db))
	{
		my $result = $db->search_person;
		my $i;
		my $i2;
		foreach $i (@$result)
		{
			$i2 = $db->fetch_person($i);
			my $iter = $peoplelistmodel->append;
			$peoplelistmodel->set($iter, 0, $i2->{'name'}, 1, $i2->{'sortkey'}, 2, $i2->{'id'});
		}
	}
}

sub add_person_entry
{
	my $rec = shift;
	my $res = $db->add_person($rec);

	&create_people_model;
	return $res;
}

sub show_work_detail
{
	my ($id) = @_;
	
	my $wd = $db->fetch_work($id);
	my $dlg;
	
	# create dialogue for editing the work
	$dlg = Gtk2::Dialog->new('Edit Work', $mainwindow, [ 'modal' ], 'Save' => 'ok', 'gtk-cancel' => 'cancel');
	
	# add a title to the dialog
	my $w = Gtk2::Label->new;
	$w->set_markup('<span size="x-large" font_weight="bold">Edit Work</span>');
	$dlg->get_content_area->pack_start($w, 0, 1, 0);

	# add title entry box
	my $tbox = Gtk2::HBox->new;
	$dlg->get_content_area->pack_start($tbox, 0, 1, 0);
	$w = Gtk2::Label->new;
	$w->set_markup('<span font-weight="bold">Title:</span>');
	$tbox->pack_start($w, 0, 1, 0);
	my $titleentry = Gtk2::Entry->new;
	$titleentry->set_text($wd->{'title'}) if defined($wd->{'title'});
	$tbox->pack_start_defaults($titleentry);
	
	$dlg->get_content_area->pack_start(Gtk2::HSeparator->new, 0, 1, 0);
	# contributors
	$w = Gtk2::Label->new;
	$w->set_markup('<span font-weight="bold" size="large">Contributors</span>');
	$w->set_alignment(0, 0);
	$dlg->get_content_area->pack_start($w, 0, 1, 0);
	$tbox = Gtk2::Table->new(1, 0);
	$dlg->get_content_area->pack_start($tbox, 0, 1, 0);
	my @contribentry = ();
	my @contribtypes = ();
	my $c = 0;
	my $v;
	foreach $v (@{ $wd->{'contribs'} })
	{
		my $ce = Gtk2::Entry->new;
		my $cec = Gtk2::EntryCompletion->new;
		$cec->set_model($state{'peoplelistmodel'});
		$cec->set_text_column(0);
		$ce->set_completion($cec);
		$ce->set_text($v->{'name'}) if defined($v->{'name'});
		$tbox->attach($ce, 0, 1, $c, $c + 1, [ 'shrink' ], ['shrink'], 0, 0);
		$contribentry[$c] = $ce;
		$ce = Gtk2::ComboBox->new_text;
		$ce->append_text('Author');
		$ce->append_text('Editor');
		$tbox->attach($ce, 1, 2, $c, $c + 1, [ 'shrink' ], [ 'shrink' ], 0, 0);
		$ce->set_active(0);
		$ce->set_active(1) if ($v->{'type'} eq 'editor');
		$contribtypes[$c] = $ce;
		$ce = Gtk2::Button->new_with_label('Remove');
		$ce->signal_connect('clicked' => sub {
				my ($btn, $dta) = @_;
				my $a1 = $dta->[0];
				my $a2 = $dta->[1];
				my $c = $dta->[2];
				$a1->[$c]->destroy;
				$a2->[$c]->destroy;
				$a1->[$c] = undef;
				$a2->[$c] = undef;
				$btn->destroy;
			}, [ \@contribentry, \@contribtypes, $c ]
		);
		$tbox->attach($ce, 2, 3, $c, $c + 1, [ 'shrink' ], [ 'shrink' ], 0, 0);
		$c++;
	}
	my $ce = Gtk2::Button->new_with_label('Add Contributor');
	$ce->signal_connect('clicked' => sub {
				my ($btn, $dta) = @_;
				my $ct = $dta->[1];
				my $entries = $dta->[0];
				my $ce = Gtk2::Entry->new;
				my $cec = Gtk2::EntryCompletion->new;
				my $tbox = $dta->[2];
				my $c = @{$entries};
				$cec->set_model($state{'peoplelistmodel'});
				$cec->set_text_column(0);
				$ce->set_completion($cec);
				$entries->[$c] = $ce;
				$$tbox->attach($ce, 0, 1, $c, $c + 1, [ 'shrink' ], [ 'shrink' ], 0, 0);
				$ce = Gtk2::ComboBox->new_text;
				$ce->append_text('Author');
				$ce->append_text('Editor');
				$$tbox->attach($ce, 1, 2, $c, $c + 1, [ 'shrink' ], [ 'shrink' ], 0, 0);
				$ce->set_active(0);
				$ct->[$c] = $ce;
				$ce = Gtk2::Button->new_with_label('Remove');
				$ce->signal_connect('clicked' => sub {
						my ($btn, $dta) = @_;
						my $a1 = $dta->[0];
						my $a2 = $dta->[1];
						my $c = $dta->[2];
						$a1->[$c]->destroy;
						$a2->[$c]->destroy;
						$a1->[$c] = undef;
						$a2->[$c] = undef;
						$btn->destroy;
					}, [ $entries, $ct, $c ]
				);
				$$tbox->attach($ce, 2, 3, $c, $c + 1, [ 'shrink' ], [ 'shrink' ], 0, 0);
				
				# replace button and attach new fields
				$$tbox->remove($btn);
				$$tbox->attach($btn, 0, 3, $c + 1, $c + 2, [ 'shrink' ], [ 'shrink' ], 0, 0);
				$$tbox->show_all;
				
			}, [ \@contribentry, \@contribtypes, \$tbox ]
		);
	$tbox->attach($ce, 0, 3, $c, $c + 1, [ 'shrink' ], [ 'shrink' ], 0, 0);

	$dlg->get_content_area->pack_start(Gtk2::HSeparator->new, 0, 1, 0);
	# create the copy information
	my @copylist = ();
	$w = Gtk2::Label->new;
	$w->set_markup('<span font_weight="bold" size="large">Copies</span>');
	$w->set_alignment(0, 0);
	$dlg->get_content_area->pack_start($w, 0, 1, 0);
	$c = 0;
	foreach $v (@{ $wd->{'copies'} })
	{
		my $tbox2 = Gtk2::Table->new(0, 0);
		$w = Gtk2::Label->new;
		$w->set_markup('<span font_weight="bold">Copy</span>');
		$tbox2->attach($w, 0, 2, 0, 1, [ 'shrink' ], [ 'shrink' ], 0, 0);
		
		$w = Gtk2::Label->new('ISBN:');
		$tbox2->attach($w, 0, 1, 1, 2, [ 'shrink' ], [ 'shrink' ], 0, 0);
		
		$w = Gtk2::Entry->new;
		$w->set_text($v->{'isbn'});
		$copylist[$c]->{'isbn'} = $w;
		$tbox2->attach($w, 1, 2, 1, 2, [ 'shrink' ], [ 'shrink' ], 0, 0);
		
		$w = Gtk2::Label->new('LWID:');
		$tbox2->attach($w, 0, 1, 2, 3, [ 'shrink' ], [ 'shrink' ], 0, 0);
		
		$w = Gtk2::Entry->new;
		$w->set_text($v->{'lwid'});
		$copylist[$c]->{'lwid'} = $w;
		$tbox2->attach($w, 1, 2, 2, 3, [ 'shrink' ], [ 'shrink' ], 0, 0);
		
		$copylist[$c]->{'tbox'} = $tbox2;
		
		$w = Gtk2::Button->new_with_label('Remove Copy');
		$w->signal_connect('clicked' => sub {
				my ($btn, $dta) = @_;
				my $copylist = $dta->[0];
				my $c = $dta->[1];
				my $tbox = $copylist->[$c]->{'tbox'};
				$tbox->destroy;
				$copylist->[$c] = undef;
			}, [ \@copylist, $c ]);
		$tbox2->attach($w, 2, 3, 1, 2, ['shrink'], ['shrink'], 0, 0);
		$c++;
		$dlg->get_content_area->pack_start($tbox2, 0, 1, 0);
	}
	$ce = Gtk2::Button->new_with_label('Add Copy');
	$ce->signal_connect('clicked' => sub {
				my ($btn, $dta) = @_;
				my $dlg = $dta->[1];
				my $copylist = $dta->[0];
				my $c = @{$copylist};
				
				my $tbox2 = Gtk2::Table->new(0, 0);
				my $w = Gtk2::Label->new;
				$w->set_markup('<span font_weight="bold">Copy</span>');
				$tbox2->attach($w, 0, 2, 0, 1, [ 'shrink' ], [ 'shrink' ], 0, 0);
		
				$w = Gtk2::Label->new('ISBN:');
				$tbox2->attach($w, 0, 1, 1, 2, [ 'shrink' ], [ 'shrink' ], 0, 0);
		
				$w = Gtk2::Entry->new;
				$copylist[$c]->{'isbn'} = $w;
				$tbox2->attach($w, 1, 2, 1, 2, [ 'shrink' ], [ 'shrink' ], 0, 0);
		
				$w = Gtk2::Label->new('LWID:');
				$tbox2->attach($w, 0, 1, 2, 3, [ 'shrink' ], [ 'shrink' ], 0, 0);
		
				$w = Gtk2::Entry->new;
				$copylist[$c]->{'lwid'} = $w;
				$tbox2->attach($w, 1, 2, 2, 3, [ 'shrink' ], [ 'shrink' ], 0, 0);

				$copylist[$c]->{'tbox'} = $tbox2;

				$w = Gtk2::Button->new_with_label('Remove Copy');
				$w->signal_connect('clicked' => sub {
						my ($btn, $dta) = @_;
						my $copylist = $dta->[0];
						my $c = $dta->[1];
						my $tbox = $copylist->[$c]->{'tbox'};
						$tbox->destroy;
						$copylist->[$c] = undef;
					}, [ \@copylist, $c ]);
				$tbox2->attach($w, 2, 3, 1, 2, ['shrink'], ['shrink'], 0, 0);
		
				# remove add button
				$$dlg->get_content_area->remove($btn);
				
				# add new copy area
				$$dlg->get_content_area->pack_start($tbox2, 0, 1, 0);
				
				# add back add button
				$$dlg->get_content_area->pack_start($btn, 0, 0, 0);
				
				$$dlg->show_all;
			}, [ \@copylist, \$dlg ]
		);
	$dlg->get_content_area->pack_start($ce, 0, 0, 0);
	#->attach($ce, 0, 3, $c, $c + 1, [ 'shrink' ], [ 'shrink' ], 0, 0);
	
DUMMY:	while (1)
	{
		$dlg->show_all;
		my $res = $dlg->run;

		if ($res eq 'ok')
		{
			# update the work
#			print "Saving work...\n";
			# first, build an updated data structure
			my $neww = {};
			$neww->{'title'} = $titleentry->get_text;
		
			# contributors
			my $c = 0;
			$neww->{'contribs'} = [];
			my $cnt;
			for ($cnt = 0; $cnt <= $#contribentry; $cnt += 1)
			{
				next if !defined($contribentry[$cnt]);
				my $name = $contribentry[$cnt]->get_text;
				my $type = $contribtypes[$cnt]->get_active_text;
				my $persons = $db->search_person({ 'name' => $name });
				my $pid;
				if ($#$persons != 0)
				{
					# handle a non-existent contributor
					$pid = &query_person($name);
					next DUMMY if !defined($pid);
				}
				else
				{
					$pid = $persons->[0];
				}
				$type = lc($type);
				$neww->{'contribs'}->[$c] = { 'type' => $type, 'id' => $pid, 'name' => $name };
				$c += 1;
			}
			
			$neww->{'copies'} = [];
			$c = 0;
			foreach my $copy (@copylist)
			{
				my $f = 0;
				next if !defined($copy->{'lwid'});
				my $lwid = $copy->{'lwid'}->get_text;
				my $isbn = $copy->{'isbn'}->get_text;
				next if ($lwid eq '' || $isbn eq '');
				{
					$neww->{'copies'}->[$c] = {'lwid' => $lwid, 'isbn' => $isbn};
					$c++;
				}
			}
			if ($c == 0)
			{
				# need to put up dialogue box here
				next;
			}
			
			$neww->{'id'} = $id if defined($id);
		
			# now fuzz-check it

			# save it to the data store
			$db->save_work($neww);
			
			# Q&D: update data model
			&create_work_model;
		}
		last;
	}
	$dlg->destroy;
}

sub query_person
{
	my $name = shift;
	
	# create dialogue for creating a new person record
	my $dlg = Gtk2::Dialog->new('Create Person', $mainwindow, [ 'modal' ], 'Save' => 'ok', 'gtk-cancel' => 'cancel');
	
	# add a title to the dialog
	my $w = Gtk2::Label->new;
	$w->set_markup('<span size="x-large" font_weight="bold">Create Person</span>');
	$dlg->get_content_area->pack_start($w, 0, 1, 0);

	# add title entry box
	my $tbox = Gtk2::HBox->new;
	$dlg->get_content_area->pack_start($tbox, 0, 1, 0);
	$w = Gtk2::Label->new;
	$w->set_markup('<span font-weight="bold">Name:</span>');
	$tbox->pack_start($w, 0, 1, 0);
	my $nameentry = Gtk2::Entry->new;
	$nameentry->set_text($name);
	$tbox->pack_start_defaults($nameentry);
	
	$tbox = Gtk2::HBox->new;
	$dlg->get_content_area->pack_start($tbox, 0, 1, 0);
	$w = Gtk2::Label->new;
	$w->set_markup('<span font-weight="bold">Sort Key:</span>');
	$tbox->pack_start($w, 0, 1, 0);
	my $skentry = Gtk2::Entry->new;
	$tbox->pack_start_defaults($skentry);

	$dlg->show_all;
	my $res = $dlg->run;
	
	if ($res eq 'ok')
	{
		my $name = $nameentry->get_text;
		my $sortkey = $skentry->get_text;
		# add person entry
		$dlg->destroy;
		my $rid = &add_person_entry({'name' => $name, 'sortkey' => $sortkey});
#		print "Person ID: " . Dumper($rid) . "\n\n";
		&create_people_model;
		return $rid;
	}
	
	$dlg->destroy;
	return undef;	
}
